@@ -1,6 +1,73 @@
-#' Calculate Harell's Concordance index
+utils::globalVariables(c("PredictionSurv"))
+#' Calculate integrated metrics based on time-dependent metrics.
 #'
-#' A function to calculate Harells' concordance index of a survival model.
+#' This function allows for creating a function for calculation of integrated metrics based on a time dependent metric. A possibility to cut off the data at certain quantiles is implemented, as well as weighting the integrated metric by max time and marginal survival function \[[1](https://onlinelibrary.wiley.com/doi/abs/10.1002/%28SICI%291097-0258%2819990915/30%2918%3A17/18%3C2529%3A%3AAID-SIM274%3E3.0.CO%3B2-5)\]
+#'
+#' @param loss_function - A time dependent loss function taking arguments (y_true, risk, surv, times)
+#'
+#' @param ... - other parameters, currently ignored
+#' @param normalization - either NULL, "t_max" or "survival". Decides what kind of weighting should be applied to the integrated metric. If "t_max", then the integral is calculated using dw(t) where w(t) = t/t_max. If "survival", then the integral is calculated using dw(t) where w(t) = (1 - S(t))/(1 - S(t_max)) and S(t) denotes the estimated marginal survival function. If NULL (default), the integral is calculated using dt.
+#' @param max_quantile - a number from the interval (0,1]. The integral will be calculated only up to the time value of `quantile(max_quantile)` of the observed event/censoring times in `y_true`.
+#'
+#' @return a function that can be used to calculate metrics (with parameters `y_true`, `risk`, `surv`, and `times`)
+#'
+#' @section References:
+#' - \[1\] Graf, Erika, et al. ["Assessment and comparison of prognostic classification schemes for survival data."](https://onlinelibrary.wiley.com/doi/abs/10.1002/%28SICI%291097-0258%2819990915/30%2918%3A17/18%3C2529%3A%3AAID-SIM274%3E3.0.CO%3B2-5) Statistics in Medicine 18.17‐18 (1999): 2529-2545.
+#'
+#' @export
+loss_integrate <- function(loss_function, ..., normalization = NULL , max_quantile = 1){
+
+    integrated_loss_function <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL){
+
+        quantile_mask <- (times <= quantile(y_true[,1],max_quantile))
+        times <- times[quantile_mask]
+        surv <- surv[,quantile_mask]
+
+        loss_values <- loss_function(y_true = y_true, risk = risk, surv = surv,times = times)
+
+        na_mask <- (!is.na(loss_values))
+
+        times <- times[na_mask]
+        loss_values <- loss_values[na_mask]
+        surv <- surv[na_mask]
+
+        n <- length(loss_values)
+        # integral using trapezoid method
+
+        if (is.null(normalization)){
+            tmp <- (loss_values[1:(n - 1)] + loss_values[2:n]) * diff(times) / 2
+            integrated_metric <- cumsum(c(0, tmp))[length(tmp)+1] / (max(times) - min(times))
+            return(integrated_metric)
+        }
+        else if (normalization == "t_max") {
+            tmp <- (loss_values[1:(n - 1)] + loss_values[2:n]) * diff(times) / 2
+            integrated_metric <- cumsum(c(0, tmp))[length(tmp)+1]
+            return(integrated_metric/max(times))
+        } else if (normalization == "survival"){
+
+            kaplan_meier <- survfit(y_true~1)
+            estimator <- transform_to_stepfunction(kaplan_meier, eval_times = sort(unique(y_true[,1])), type = "survival")
+
+            dwt <- 1 - estimator(times)
+
+            tmp <- (loss_values[1:(n - 1)] + loss_values[2:n]) * diff(dwt) / 2
+            integrated_metric <- cumsum(c(0, tmp))[length(tmp)+1]
+            return(integrated_metric/(1 - estimator(max(times))))
+        }
+        else stop("normalization should be either NULL, `t_max` or `survival`")
+    }
+
+    attr(integrated_loss_function, "loss_type") <- "integrated"
+    attr(integrated_loss_function, "loss_name") <- paste("Integrated", attr(loss_function, "loss_name"))
+    return(integrated_loss_function)
+}
+
+
+
+
+#' Compute the Harrell's Concordance index
+#'
+#' A function to compute the Harrell's concordance index of a survival model.
 #'
 #'
 #' @param y_true a `survival::Surv` object containing the times and statuses of observations for which the metric will be evaluated
@@ -10,6 +77,9 @@
 #'
 #' @return numeric from 0 to 1, higher values indicate better performance
 #'
+#' @section References:
+#' - \[1\] Harrell, F.E., Jr., et al. ["Regression modelling strategies for improved prognostic prediction."](https://onlinelibrary.wiley.com/doi/10.1002/sim.4780030207) Statistics in Medicine 3.2 (1984): 143-152.
+#'
 #' @rdname c_index
 #' @seealso [loss_one_minus_c_index()]
 #'
@@ -28,7 +98,7 @@
 #' risk <- coxph_explainer$predict_function(coxph_explainer$model, coxph_explainer$data)
 #' c_index(y_true = coxph_explainer$y, risk = risk)
 #' }
-#' @import survival
+#'
 #' @export
 c_index <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
 
@@ -49,6 +119,8 @@ c_index <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
 
 }
 attr(c_index, "loss_name") <- "C-index"
+attr(c_index, "loss_type") <- "risk-based"
+
 
 #' Calculate the Concordance index loss
 #'
@@ -58,6 +130,9 @@ attr(c_index, "loss_name") <- "C-index"
 #'
 #' @return numeric from 0 to 1, lower values indicate better performance
 #'
+#' @section References:
+#' - \[1\] Harrell, F.E., Jr., et al. ["Regression modelling strategies for improved prognostic prediction."](https://onlinelibrary.wiley.com/doi/10.1002/sim.4780030207) Statistics in Medicine 3.2 (1984): 143-152.
+#'
 #' @rdname loss_one_minus_c_index
 #' @seealso [c_index()]
 #'
@@ -81,9 +156,7 @@ loss_one_minus_c_index <- function(y_true = NULL, risk = NULL, surv = NULL, time
     1 - c_index(y_true = y_true, risk = risk, surv = surv, times = times)
 }
 attr(loss_one_minus_c_index, "loss_name") <- "One minus C-Index"
-
-
-
+attr(loss_one_minus_c_index, "loss_type") <- "risk-based"
 
 #' Calculate Brier score
 #'
@@ -97,10 +170,13 @@ attr(loss_one_minus_c_index, "loss_name") <- "One minus C-Index"
 #' @param surv a matrix containing the predicted survival functions for the considered observations, each row represents a single observation, whereas each column one time point
 #' @param times a vector of time points at which the survival function was evaluated
 #'
-#' @return numeric from 0 to 1, lower scores are better (brier score of 0.25 represents a model which returns always returns 0.5 as the predicted survival function)
+#' @return numeric from 0 to 1, lower scores are better (Brier score of 0.25 represents a model which returns always returns 0.5 as the predicted survival function)
+#'
+#' @section References:
+#' - \[1\] Brier, Glenn W. ["Verification of forecasts expressed in terms of probability."](https://journals.ametsoc.org/view/journals/mwre/78/1/1520-0493_1950_078_0001_vofeit_2_0_co_2.xml) Monthly Weather Review 78.1 (1950): 1-3.
+#' - \[2\] Graf, Erika, et al. ["Assessment and comparison of prognostic classification schemes for survival data."](https://onlinelibrary.wiley.com/doi/10.1002/(SICI)1097-0258(19990915/30)18:17/18%3C2529::AID-SIM274%3E3.0.CO;2-5) Statistics in Medicine 18.17‐18 (1999): 2529-2545.
 #'
 #' @rdname brier_score
-#'
 #' @seealso [cd_auc()]
 #'
 #' @examples
@@ -117,7 +193,6 @@ attr(loss_one_minus_c_index, "loss_name") <- "One minus C-Index"
 #' brier_score(y, surv = surv, times = times)
 #' loss_brier_score(y, surv = surv, times = times)
 #'
-#' @import survival
 #' @export
 brier_score <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
     # if times is not provided use
@@ -150,17 +225,23 @@ brier_score <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
 
 }
 attr(brier_score, "loss_name") <- "Brier score"
+attr(brier_score, "loss_type") <- "time-dependent"
 
 #' @rdname brier_score
 #' @export
 loss_brier_score <- brier_score
 attr(loss_brier_score, "loss_name") <- "Brier score"
+attr(loss_brier_score, "loss_type") <- "time-dependent"
 
 #' Calculate Cumulative/Dynamic AUC
 #'
-#' This function calculates the Cumulative/Dynamic AUC metric for a survival model.
+#' This function calculates the Cumulative/Dynamic AUC metric for a survival model. It is done using the
+#' estimator proposed proposed by Uno et al. \[[1](https://www.jstor.org/stable/27639883)\],
+#' and Hung and Chang \[[2](https://www.jstor.org/stable/41000414)\].
 #'
-#' C/D AUC is an extension of the AUC metric known from classification models. Its values represent the model's performance at specific time points. It can be integrated over the considered time point
+#' C/D AUC is an extension of the AUC metric known from classification models.
+#' Its values represent the model's performance at specific time points.
+#' It can be integrated over the considered time range.
 #'
 #' @param y_true a `survival::Surv` object containing the times and statuses of observations for which the metric will be evaluated
 #' @param risk ignored, left for compatibility with other metrics
@@ -169,6 +250,10 @@ attr(loss_brier_score, "loss_name") <- "Brier score"
 #'
 #' @return a numeric vector of length equal to the length of the times vector, each value (from the range from 0 to 1) represents the AUC metric at a specific time point, with higher values indicating better performance.
 #'
+#' @section References:
+#' - \[1\] Uno, Hajime, et al. ["Evaluating prediction rules for t-year survivors with censored regression models."](https://www.jstor.org/stable/27639883) Journal of the American Statistical Association 102.478 (2007): 527-537.
+#' - \[2\] Hung, Hung, and Chin‐Tsang Chiang. ["Optimal composite markers for time dependent receiver operating characteristic curves with censored survival data."](https://www.jstor.org/stable/41000414) Scandinavian Journal of Statistics 37.4 (2010): 664-679.
+#'
 #' @rdname cd_auc
 #' @seealso [loss_one_minus_cd_auc()] [integrated_cd_auc()] [brier_score()]
 #'
@@ -185,7 +270,6 @@ attr(loss_brier_score, "loss_name") <- "Brier score"
 #'
 #' cd_auc(y, surv = surv, times = times)
 #'
-#' @import survival
 #' @export
 cd_auc <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
 
@@ -222,6 +306,7 @@ cd_auc <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
 
 }
 attr(cd_auc, "loss_name") <- "C/D AUC"
+attr(cd_auc, "loss_type") <- "time-dependent"
 
 
 #' Calculate Cumulative/Dynamic AUC loss
@@ -232,6 +317,10 @@ attr(cd_auc, "loss_name") <- "C/D AUC"
 #'
 #' @return a numeric vector of length equal to the length of the times vector, each value (from the range from 0 to 1) represents 1 - AUC metric at a specific time point, with lower values indicating better performance.
 #'
+#' #' @section References:
+#' - \[1\] Uno, Hajime, et al. ["Evaluating prediction rules for t-year survivors with censored regression models."](https://www.jstor.org/stable/27639883) Journal of the American Statistical Association 102.478 (2007): 527-537.
+#' - \[2\] Hung, Hung, and Chin‐Tsang Chiang. ["Optimal composite markers for time‐dependent receiver operating characteristic curves with censored survival data."](https://www.jstor.org/stable/41000414) Scandinavian Journal of Statistics 37.4 (2010): 664-679.
+#'
 #' @rdname loss_one_minus_cd_auc
 #' @seealso [cd_auc()]
 #'
@@ -253,6 +342,7 @@ loss_one_minus_cd_auc <- function(y_true = NULL, risk = NULL, surv = NULL, times
     1 - cd_auc(y_true = y_true, risk = risk, surv = surv, times = times)
 }
 attr(loss_one_minus_cd_auc, "loss_name") <- "One minus C/D AUC"
+attr(loss_one_minus_cd_auc, "loss_type") <- "time-dependent"
 
 #' Calculate integrated C/D AUC
 #'
@@ -264,10 +354,13 @@ attr(loss_one_minus_cd_auc, "loss_name") <- "One minus C/D AUC"
 #' @param risk ignored, left for compatibility with other metrics
 #' @param surv a matrix containing the predicted survival functions for the considered observations, each row represents a single observation, whereas each column one time point
 #' @param times a vector of time points at which the survival function was evaluated
-#' @param auc a vector containing already calculated AUC metric at the time points specified in the times parameter. If this is provided all arguments except `times` and `auc` are ignored
 #'
 #' @return numeric from 0 to 1, higher values indicate better performance
 #'
+#' #' @section References:
+#' - \[1\] Uno, Hajime, et al. ["Evaluating prediction rules for t-year survivors with censored regression models."](https://www.jstor.org/stable/27639883) Journal of the American Statistical Association 102.478 (2007): 527-537.
+#' - \[2\] Hung, Hung, and Chin‐Tsang Chiang. ["Optimal composite markers for time‐dependent receiver operating characteristic curves with censored survival data."](https://www.jstor.org/stable/41000414) Scandinavian Journal of Statistics 37.4 (2010): 664-679.
+#'
 #' @rdname integrated_cd_auc
 #' @seealso [cd_auc()] [loss_one_minus_cd_auc()]
 #'
@@ -283,31 +376,13 @@ attr(loss_one_minus_cd_auc, "loss_name") <- "One minus C/D AUC"
 #' times <- cph_exp$times
 #' surv <- cph_exp$predict_survival_function(cph, cph_exp$data, times)
 #'
-#'
-#' # calculating directly
 #' integrated_cd_auc(y, surv = surv, times = times)
 #'
-#' # calculating based on given auc vector
-#' auc <- cd_auc(y, surv = surv, times = times)
-#' integrated_cd_auc(times = times, auc = auc)
-#'
 #' @export
-integrated_cd_auc <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL, auc = NULL) {
-    if (is.null(auc)) {
-        auc <- cd_auc(y_true, risk, surv, times)
-    }
-
-    auc <- as.numeric(auc)
-    ind_to_drop <- is.na(auc)
-    times <- times[!ind_to_drop]
-    auc <- auc[!ind_to_drop]
-    n <- length(auc)
-
-    iauc <- (auc[1:(n - 1)] + auc[2:n]) * diff(times) / 2
-
-    cumsum(c(0, iauc))[length(cumsum(c(0, iauc)))] / (max(times) - min(times))
-}
+integrated_cd_auc <- loss_integrate(cd_auc)
 attr(integrated_cd_auc, "loss_name") <- "integrated C/D AUC"
+attr(integrated_cd_auc, "loss_type") <- "integrated"
+
 
 
 #' Calculate integrated C/D AUC loss
@@ -318,6 +393,10 @@ attr(integrated_cd_auc, "loss_name") <- "integrated C/D AUC"
 #'
 #' @return numeric from 0 to 1, lower values indicate better performance
 #'
+#' #' @section References:
+#' - \[1\] Uno, Hajime, et al. ["Evaluating prediction rules for t-year survivors with censored regression models."](https://www.jstor.org/stable/27639883) Journal of the American Statistical Association 102.478 (2007): 527-537.
+#' - \[2\] Hung, Hung, and Chin‐Tsang Chiang. ["Optimal composite markers for time‐dependent receiver operating characteristic curves with censored survival data."](https://www.jstor.org/stable/41000414) Scandinavian Journal of Statistics 37.4 (2010): 664-679.
+#'
 #' @rdname loss_one_minus_integrated_cd_auc
 #' @seealso [integrated_cd_auc()] [cd_auc()] [loss_one_minus_cd_auc()]
 #'
@@ -333,19 +412,17 @@ attr(integrated_cd_auc, "loss_name") <- "integrated C/D AUC"
 #' times <- cph_exp$times
 #' surv <- cph_exp$predict_survival_function(cph, cph_exp$data, times)
 #'
-#'
 #' # calculating directly
 #' loss_one_minus_integrated_cd_auc(y, surv = surv, times = times)
 #'
-#' # calculating based on given auc vector
-#' auc <- cd_auc(y, surv = surv, times = times)
-#' loss_one_minus_integrated_cd_auc(times = times, auc = auc)
-#'
 #' @export
-loss_one_minus_integrated_cd_auc <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL, auc = NULL) {
-    1 - integrated_cd_auc(y_true = y_true, risk = risk, surv = surv, times = times, auc = auc)
+loss_one_minus_integrated_cd_auc <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL) {
+    1 - integrated_cd_auc(y_true = y_true, risk = risk, surv = surv, times = times)
 }
 attr(loss_one_minus_integrated_cd_auc, "loss_name") <- "One minus integrated C/D AUC"
+attr(loss_one_minus_integrated_cd_auc, "loss_type") <- "integrated"
+
+
 
 #' Calculate integrated Brier score
 #'
@@ -357,10 +434,13 @@ attr(loss_one_minus_integrated_cd_auc, "loss_name") <- "One minus integrated C/D
 #' @param risk ignored, left for compatibility with other metrics
 #' @param surv a matrix containing the predicted survival functions for the considered observations, each row represents a single observation, whereas each column one time point
 #' @param times a vector of time points at which the survival function was evaluated
-#' @param brier a vector containing already calculated Brier score metric at the time points specified in the times parameter. If this is provided all arguments except `times` and `brier` are ignored
 #'
 #' @return numeric from 0 to 1, lower values indicate better performance
 #'
+#' @section References:
+#' - \[1\] Brier, Glenn W. ["Verification of forecasts expressed in terms of probability."](https://journals.ametsoc.org/view/journals/mwre/78/1/1520-0493_1950_078_0001_vofeit_2_0_co_2.xml) Monthly Weather Review 78.1 (1950): 1-3.
+#' - \[2\] Graf, Erika, et al. ["Assessment and comparison of prognostic classification schemes for survival data."](https://onlinelibrary.wiley.com/doi/10.1002/(SICI)1097-0258(19990915/30)18:17/18%3C2529::AID-SIM274%3E3.0.CO;2-5) Statistics in Medicine 18.17‐18 (1999): 2529-2545.
+#'
 #' @rdname integrated_brier_score
 #' @seealso [brier_score()] [integrated_cd_auc()] [loss_one_minus_integrated_cd_auc()]
 #'
@@ -376,35 +456,60 @@ attr(loss_one_minus_integrated_cd_auc, "loss_name") <- "One minus integrated C/D
 #' times <- cph_exp$times
 #' surv <- cph_exp$predict_survival_function(cph, cph_exp$data, times)
 #'
-#'
 #' # calculating directly
 #' integrated_brier_score(y, surv = surv, times = times)
 #'
-#' # calculating based on given auc vector
-#' brier_score <- brier_score(y, surv = surv, times = times)
-#' integrated_brier_score(times = times, brier = brier_score)
-#'
 #' @export
-integrated_brier_score <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL, brier = NULL) {
-
-    if (is.null(brier)) {
-        brier <- brier_score(y_true, risk, surv, times)
-    }
-
-    brier <- as.numeric(brier)
-    ind_to_drop <- is.na(brier)
-    times <- times[!ind_to_drop]
-    brier <- brier[!ind_to_drop]
-    n <- length(brier)
-
-    ibs <- (brier[1:(n - 1)] + brier[2:n]) * diff(times) / 2
-
-    cumsum(c(0, ibs))[length(cumsum(c(0, ibs)))] / (max(times) - min(times))
-
-}
+integrated_brier_score <- loss_integrate(brier_score)
 attr(integrated_brier_score, "loss_name") <- "integrated Brier score"
 
 #' @rdname integrated_brier_score
 #' @export
 loss_integrated_brier_score <- integrated_brier_score
 attr(loss_integrated_brier_score, "loss_name") <- "integrated Brier score"
+attr(loss_integrated_brier_score, "loss_type") <- "integrated"
+
+#' Adapt mlr3proba measures for use with survex
+#'
+#' This function allows for usage of standardized measures from the mlr3proba package with `survex`.
+#'
+#' @param measure - a `MeasureSurv` object from the `mlr3proba` package, the object to adapt
+#' @param reverse - boolean, FALSE by default, whether the metric should be reversed in order to be treated as loss (for permutational variable importance we need functions with lower values indicating better performance). If TRUE, the new metric value will be (1 - metric_value)
+#' @param ... - other parameters, currently ignored
+#'
+#' @return a function with standardized parameters (`y_true`, `risk`, `surv`, `times`) that can be used to calculate loss
+#'
+#' if(FALSE){
+#'   measure <- msr("surv.calib_beta")
+#'   mlr_measure <- loss_adapt_mlr3proba(measure)
+#' }
+#'
+#' @export
+loss_adapt_mlr3proba <- function(measure, reverse = FALSE, ...){
+
+    loss_function <- function(y_true = NULL, risk = NULL, surv = NULL, times = NULL){
+
+        colnames(surv) <- times
+
+        surv_pred <- PredictionSurv$new(
+            row_ids = 1:length(y_true),
+            truth = y_true,
+            crank = risk,
+            distr = surv,
+            task = list(truth = y_true)
+        )
+
+        output <- surv_pred$score(measure)
+        names(output) <- NULL
+
+        if (reverse) output <- (1 - output)
+
+        return(output)
+    }
+
+    if (reverse) attr(loss_function, "loss_name") <- paste("one minus", measure$id)
+    else attr(loss_function, "loss_name") <- measure$id
+    attr(loss_function, "loss_type") <- "integrated"
+
+    return(loss_function)
+}
